from django.conf import settings
from django.forms.util import flatatt
from django.forms.widgets import Textarea
from django.template.loader import render_to_string
from django.utils.safestring import mark_safe


class CKEditorWidget(Textarea):
    def render(self, name, value=None, attrs=None):
        """
        Generate the markup for the HTML editor.
        """
        
        # If the value wasn't specified in the calls that led to this render(),
        # set a blank string.
        if value is None: value = ''
        # Set attributes how we want them, so we can call out all the relevant
        # text areas.
        if attrs is None:
            attrs = {}
        attrs.update({'class': 'replace-with-ckeditor'})
        # Build the final attributes.
        final_attrs = self.build_attrs(attrs, name=name)
        
        return mark_safe(render_to_string(
            'flexible_ckeditor/widget.html',
            {
                'content': value,
                'attrs': final_attrs,
                'attr_string': flatatt(final_attrs),
            }
        ))



from django.db import models
from django.utils.translation import ugettext as _

from flexible_content.models import BaseItem

from .fields import CKEditorField


class CKEditorText(BaseItem):
    html = CKEditorField()

    class FlexibleContentInfo:
        description = _("Add text that's saved as HTML, which supports "
                        "basic formatting.")
        name = "Text"
        type_slug = 'ckeditor-text'

    class Meta:
        verbose_name = _("Text")



from django.conf import settings
from django.test import TestCase
from django.test.utils import override_settings
from django.utils.unittest import skip

from mock_project.myapp.models import BlogPost

from .fields import CKEditorField, CKEditorFormField
from .models import CKEditorText

# Define a couple scenarios for INSTALLED_APPS.
INSTALLED_APPS__WITH_SITES = (settings.INSTALLED_APPS +
                              ('django.contrib.sites',))
INSTALLED_APPS__WITHOUT_SITES = [a for a in settings.INSTALLED_APPS
                                 if a!='django.contrib.sites']

# Define a couple scenarios for FLEXIBLE_CONTENT as well.
_ITEM_TYPES = (
    'flexible_ckeditor.CKEditorText',
    'default_item_types.Image',
)
FLEXIBLE_CONTENT__NO_DOMAIN = {
    'ITEM_TYPES': _ITEM_TYPES,
}
FLEXIBLE_CONTENT__WITH_SITE_DOMAIN = {
    'ITEM_TYPES': _ITEM_TYPES,
    'ckeditor': {
        'site_domain': 'www.mydomain.com',
    },
}
FLEXIBLE_CONTENT__NO_DOMAIN_CLEAN = {
    'ITEM_TYPES': _ITEM_TYPES,
    'ckeditor': {
        'clean_domains': False,
    },
}


class ModelFieldUnitTest(TestCase):
    fixtures = ['initial_data.json']

    def setUp(self):
        # Store the only post here on the testcase instance.
        self.post = BlogPost.objects.get(pk=1)
        self.ckeditor_value = self.post.items[1].html
        # Store the CKEditorField as well.
        self.model_field = (self.post.items[1]._meta.
                            get_field_by_name('html')[0])

    def test_formfield(self):
        """
        Given the field on the model, we should be able to get our formfield.
        """
        self.assertIsInstance(self.model_field.formfield(), CKEditorFormField)

    @override_settings(INSTALLED_APPS=INSTALLED_APPS__WITH_SITES)
    def test_get_forbidden_domains__sites(self):
        """
        Make sure that we can get forbidden domains from the Sites framework.
        """
        self.assertEqual(set(self.model_field.get_forbidden_domains()),
                         set(['mydomain.com', 'dev.mydomain.com']))

    @override_settings(INSTALLED_APPS=INSTALLED_APPS__WITHOUT_SITES,
                       FLEXIBLE_CONTENT=FLEXIBLE_CONTENT__WITH_SITE_DOMAIN)
    def test_get_forbidden_domains__settings(self):
        """
        Make sure that we can get forbidden domains from FLEXIBLE_CONTENT
        settings.
        """
        self.assertEqual(set(self.model_field.get_forbidden_domains()),
                         set(['mydomain.com']))

    @override_settings(INSTALLED_APPS=INSTALLED_APPS__WITHOUT_SITES,
                       FLEXIBLE_CONTENT=FLEXIBLE_CONTENT__NO_DOMAIN)
    def test_get_forbidden_domains__none(self):
        """
        Ensure that, with no sites and no FLEXIBLE_CONTENT domain setting,
        we don't forbid any domains.
        """
        self.assertEqual(len(self.model_field.get_forbidden_domains()), 0)

    @override_settings(INSTALLED_APPS=INSTALLED_APPS__WITH_SITES)
    def test_get_domain_agnostic_value(self):
        """
        For a given list of domains, ensure they're removed from the HTML.
        """
        # Get the Site domains. There should be two of them - the production
        # and dev.
        forbidden_domains = self.model_field.get_forbidden_domains()
        self.assertEqual(len(forbidden_domains), 2)

        # Get the domain agnostic version of the string and make sure it doesn't contain the forbidden domains.
        clean_value = (self.model_field.
                       get_domain_agnostic_value(self.ckeditor_value,
                                                 forbidden_domains))
        for d in forbidden_domains:
            self.assertNotIn(d, clean_value,
                             msg="Found {domain} in what was supposed to be "
                             "the domain-agnostic/cleaned HTML.".
                             format(domain=d))

        # Ensure the safe domain is still in there.
        self.assertIn('externaldomain.com', clean_value,
                      msg="An external domain was also stripped out of the "
                      "domain-agnostic/cleaned HTML!")

    @override_settings(INSTALLED_APPS=INSTALLED_APPS__WITH_SITES)
    def test_clean__from_sites(self):
        """
        When we do find sites, ensure they're properly removed.
        """
        clean_value = self.model_field.clean(self.ckeditor_value, self.post)
        self.assertNotIn('dev.mydomain.com', clean_value)

    @override_settings(INSTALLED_APPS=INSTALLED_APPS__WITH_SITES,
                       FLEXIBLE_CONTENT=FLEXIBLE_CONTENT__NO_DOMAIN_CLEAN)
    def test_clean__when_clean_domains_disabled(self):
        """
        When they have domain cleaning disabled, ensure we honor that.
        """
        clean_value = self.model_field.clean(self.ckeditor_value, self.post)
        self.assertIn('dev.mydomain.com', clean_value)


class FormFieldUnitTest(TestCase):
    fixtures = ['initial_data.json']

    def setUp(self):
        # Store the only post here on the testcase instance.
        self.post = BlogPost.objects.get(pk=1)
        self.ckeditor_value = self.post.items[1].html
        # Store the CKEditorField as well.
        self.model_field = (self.post.items[1]._meta.
                            get_field_by_name('html')[0])

    def test_render(self):
        widget = self.model_field.formfield().widget
        rendered_widget = widget.render('test_field_name')
        # Make sure the Javascript shows.
        self.assertIn('script', rendered_widget)
        self.assertIn('CKEDITOR', rendered_widget)
        # Make sure we have the test field name.
        self.assertIn('test_field_name', rendered_widget)



from distutils.core import setup
from setuptools import find_packages

# Find all packages here and below (should find ipanema and its children).
packages = find_packages()

setup(name='django-flexible-content-ckeditor',
      description="Rich text (WYSIWYG) support for django-flexible-content",
      url='https://github.com/hjkelly/django-flexible-content-ckeditor',
      maintainer='Harrison Kelly',
      maintainer_email='hjkelly@gmail.com',
      version='1.0.0',
      packages=packages,
      include_package_data=True,
      install_requires=['django-flexible-content'])



import re

from django.conf import settings
from django.contrib.sites.models import Site
from django.db.models.fields import Field, TextField
from django.forms.fields import CharField

from .widgets import CKEditorWidget


class CKEditorFormField(CharField):
    def __init__(self, *args, **kwargs):
        kwargs['widget'] = CKEditorWidget
        super(CKEditorFormField, self).__init__(*args, **kwargs)


class CKEditorField(TextField):
    def formfield(self, **kwargs):
        """
        Ensure that the form field is our CKEditor one.
        """
        kwargs['form_class'] = CKEditorFormField
        return super(CKEditorField, self).formfield(**kwargs)

    def clean(self, value, model_instance):
        """
        Prevent the site's domain from being stored in the database, so that
        it's portable between different environments.
        """
        # Check their config for a preference on whether or not their domains
        # are cleaned... default to True.
        try:
            clean_domains = (settings.FLEXIBLE_CONTENT['ckeditor']
                             ['clean_domains'])
        except (AttributeError, KeyError):
            clean_domains = True

        if clean_domains:
            # Find out what domains we shouldn't allow.
            forbidden_domains = self.get_forbidden_domains()
            # Remove them and store the cleaned contents of the field.
            value = self.get_domain_agnostic_value(value, forbidden_domains)
        return value

    def get_forbidden_domains(self):
        """
        Based on the project's configuration, figure out what domains shouldn't
        be allowed as fully-qualified URLs. Strip off the 'www.' prefix if it
        has it.
        """
        forbidden_domains = []

        # Are they using Django's sites framework?
        if 'django.contrib.sites' in settings.INSTALLED_APPS:
            # Make note of all the domains in the database.
            forbidden_domains = [s.domain for s in Site.objects.all()]
        # Otherwise, check the config for this app.
        else:
            try:
                site_domain = (settings.
                               FLEXIBLE_CONTENT['ckeditor']['site_domain'])
            except (AttributeError, KeyError):
                pass
            else:
                forbidden_domains = [site_domain]

        # Strip leading 'www.' if it has one.
        cleaned_forbidden_domains = []
        for f in forbidden_domains:
            if f.startswith('www.'):
                f = f[4:]
            cleaned_forbidden_domains.append(f)

        return cleaned_forbidden_domains

    def get_domain_agnostic_value(self, value, domains):
        """
        For the given HTML, remove fully-qualified references to one or more
        domains.
        """
        # Don't allow src or href attributes to contain fully-qualified URLs
        # to those domains
        pattern_template = r'(?P<attr>(src|href))=(?P<quotechar>("|\'))(https?:)?//(www.)?%s(:\d+)?/?'

        # Loop through each domain and remove references to it.
        for domain in domains:
            value = re.sub(pattern_template % domain,
                           '\g<attr>=\g<quotechar>/', unicode(value))
        return value
